@@ -21,6 +21,8 @@ SPDX-License-Identifier: MIT
 
 #include "llvm/IR/Attributes.h"
 #include <llvmWrapper/IR/Function.h>
+#include <llvmWrapper/IR/Type.h>
+
 #include "llvmWrapper/IR/InstrTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
 
@@ -265,6 +267,52 @@ ModulePass *createProcessFuncAttributesPass() { return new ProcessFuncAttributes
 
 extern bool isSupportedAggregateArgument(Argument *arg);
 
+static bool deduceIfUsesImageParams(Function *F, ModuleMetaData *ModMD) {
+  auto It = ModMD->FuncMD.find(F);
+  if (It != ModMD->FuncMD.end()) {
+    const FunctionMetaData &FuncMD = It->second;
+    IGC_ASSERT_MESSAGE(FuncMD.m_OpenCLArgBaseTypes.size() == F->arg_size(),
+                       "m_OpenCLArgBaseTypes and function argument size mismatch!");
+
+    for (const auto &BaseTy : FuncMD.m_OpenCLArgBaseTypes) {
+      if (BaseTy.find("image") != std::string::npos)
+        return true;
+    }
+  }
+
+  MDNode *Hints = F->getMetadata("non_kernel_arg_type_hints");
+  if (Hints) {
+    for (auto &Operand : Hints->operands())
+      if (MDString *MDStr = dyn_cast<MDString>(Operand))
+        if (MDStr->getString().contains_insensitive("image"))
+          return true;
+  }
+
+  // SYCL/DPC++ generates image accessor helpers that take image parameters indirectly (through a class referenced
+  // using a pointer). With typed pointers enabled, the pointer type is inspected and the helpers are inlined. This
+  // is not possible with opaque pointers and in this case the parameter types must be deduced based on the mangled
+  // function name.
+  auto IsSyclImageAccessor = [](StringRef Name) {
+    return Name.startswith("_Z") && Name.contains("sycl") && Name.contains("accessor") && Name.contains("image");
+  };
+
+  if (IsSyclImageAccessor(F->getName()))
+    return true;
+
+  for (auto &BB : *F) {
+    for (auto &I : BB) {
+      if (CallInst *CI = dyn_cast<CallInst>(&I)) {
+        if (Function *Callee = CI->getCalledFunction()) {
+          if (IsSyclImageAccessor(Callee->getName()))
+            return true;
+        }
+      }
+    }
+  }
+
+  return false;
+}
+
 // Only pointer, struct and array types are considered. E.g. vector type
 // cannot contain opaque subtypes, function type may contain but ignored.
 static void getBuiltinType(Type *T, SmallPtrSetImpl<Type *> &BuiltinTypes) {
@@ -276,7 +324,7 @@ static void getBuiltinType(Type *T, SmallPtrSetImpl<Type *> &BuiltinTypes) {
         getBuiltinType(*I, BuiltinTypes);
       }
     }
-  } else if (T->isPointerTy() && !T->isOpaquePointerTy()) {
+  } else if (T->isPointerTy() && !IGCLLVM::isPointerTy(T)) {
     return getBuiltinType(IGCLLVM::getNonOpaquePtrEltTy(cast<PointerType>(T)), BuiltinTypes);
   }
 #if LLVM_VERSION_MAJOR >= 16
@@ -409,6 +457,30 @@ static void addAlwaysInlineForImageBuiltinUserFunctions(Module &M) {
   }
 }
 
+static void collectGlobalAnnotationsPointers(const Module &M, DenseSet<const Value *> &GlobalAnnotations) {
+  auto *GA = M.getGlobalVariable("llvm.global.annotations");
+  if (!GA)
+    return;
+
+  auto *CA = dyn_cast<ConstantArray>(GA->getInitializer());
+  if (!CA)
+    return;
+
+  for (Value *Op : CA->operands()) {
+    auto *Annot = dyn_cast<ConstantStruct>(Op);
+    if (!Annot)
+      continue;
+
+    // In case of opaque pointers add whole struct, since it
+    // will be visible as function user, otherwise add bitcast
+    if (Annot->getOperand(0)->getType()->isOpaquePointerTy()) {
+      GlobalAnnotations.insert(Annot);
+    } else {
+      GlobalAnnotations.insert(Annot->getOperand(0));
+    }
+  }
+}
+
 bool ProcessFuncAttributes::runOnModule(Module &M) {
   MetaDataUtilsWrapper &mduw = getAnalysis<MetaDataUtilsWrapper>();
   MetaDataUtils *pMdUtils = mduw.getMetaDataUtils();
@@ -434,6 +506,9 @@ bool ProcessFuncAttributes::runOnModule(Module &M) {
     }
   }
 
+  DenseSet<const Value *> GlobalAnnotationsPtr;
+  collectGlobalAnnotationsPointers(M, GlobalAnnotationsPtr);
+
   auto SetNoInline = [](Function *F) {
     F->addFnAttr(llvm::Attribute::NoInline);
     F->removeFnAttr(llvm::Attribute::AlwaysInline);
@@ -447,8 +522,8 @@ bool ProcessFuncAttributes::runOnModule(Module &M) {
       // builtins should not be externally linked, they will always be resolved by IGC
       return !(F->hasFnAttribute("OclBuiltin") || F->getName().startswith("__builtin_") ||
                F->getName().startswith("__igcbuiltin_") || F->getName().startswith("llvm.") ||
-               F->getName().equals("printf") || Regex("^_Z[0-9]+__spirv_").match(F->getName()) ||
-               Regex("^_Z[0-9]+__builtin_spirv").match(F->getName()));
+               F->getName().equals("printf") || Regex("^_Z[0-9]+__builtin_bf16").match(F->getName()) ||
+               Regex("^_Z[0-9]+__spirv_").match(F->getName()) || Regex("^_Z[0-9]+__builtin_spirv").match(F->getName()));
     }
     return false;
   };
@@ -597,6 +672,11 @@ bool ProcessFuncAttributes::runOnModule(Module &M) {
         // Set the indirect flag if the function's address is taken by a non-call instruction.
         for (auto u = F->user_begin(), e = F->user_end(); u != e; u++) {
           CallInst *call = dyn_cast<CallInst>(*u);
+
+          // If user is function bitcast or function ptr in @llvm.global.annotations we should skip it.
+          if (GlobalAnnotationsPtr.find(*u) != GlobalAnnotationsPtr.end())
+            continue;
+
           if (!call || IGCLLVM::getCalledValue(call) != F) {
             isIndirect = true;
             break;
@@ -654,12 +734,18 @@ bool ProcessFuncAttributes::runOnModule(Module &M) {
       // Curently, ExtensionArgAnalysis assumes that all functions with image arguments
       // to be inlined. We add always inline for such cases.
       for (auto &arg : F->args()) {
+        // TODO: Remove following the migration to opaque pointers. The following method won't be able to recognize
+        // builtin types. TargetExtTy information is lost at this point (following the retyping in PreprocessSPVIR) and
+        // type hints in the metadata should be used instead, see deduceIfUsesImageParams below.
         if (containsImageType(arg.getType())) {
           mustAlwaysInline = true;
           break;
         }
       }
 
+      if (deduceIfUsesImageParams(F, modMD))
+        mustAlwaysInline = true;
+
       if (pCtx->m_hasDPEmu && !mustAlwaysInline && !isKernel) {
         // Prefer stackcall if a func has double operations
         bool isSet = false;
@@ -752,7 +838,7 @@ bool ProcessFuncAttributes::runOnModule(Module &M) {
         if (shouldAlwaysInline) {
           if ((IGC_IS_FLAG_ENABLED(ControlKernelTotalSize) || IGC_IS_FLAG_ENABLED(ControlUnitSize)) &&
               efs.shouldEnableSubroutine() && efs.isTrimmedFunction(F)) {
-            if (IGC_IS_FLAG_ENABLED(AddNoInlineToTrimmedFunctions)) {
+            if (IGC_IS_FLAG_ENABLED(AddNoInlineToTrimmedFunctions) || efs.isLargeKernelThresholdExceeded()) {
               SetNoInline(F);
             }
           } else {
