@@ -9,6 +9,7 @@ SPDX-License-Identifier: MIT
 #pragma once
 
 #include "RTStackFormat.h"
+#include "RTStackReflectionIRBG/RTStackReflectionBuilder.h"
 #include "common/IGCIRBuilder.h"
 #include "Compiler/CodeGenPublicEnums.h"
 #include "Compiler/CodeGenPublic.h"
@@ -23,8 +24,12 @@ SPDX-License-Identifier: MIT
 
 namespace llvm {
 
-class RTBuilder : public IGCIRBuilder<> {
+class RTBuilder : public IGCIRBuilder<>, public RTStackReflectionBuilder<RTBuilder> {
+  // Allow the CRTP base class to access private members
+  friend class RTStackReflectionBuilder<RTBuilder>;
+
 public:
+  const IGC::CodeGenContext &getCtx() const { return Ctx; }
   // Here are more specialized values that enforce more type safety
   // between the accessor methods of RTBuilder at the C++ level to avoid
   // passing the wrong values in.
@@ -183,6 +188,11 @@ public:
   RTBuilder(const RTBuilder &) = delete;
   RTBuilder &operator=(const RTBuilder &) = delete;
 
+  inline Value *getMaxSimdSize(void) { return this->getInt32(Ctx.platform.getMaxSimdSize()); }
+
+  inline Value *getMaxThreadsPerEU(void) {
+    return this->getInt32(Ctx.platform.GetGTSystemInfo().ThreadCount / Ctx.platform.GetGTSystemInfo().EUCount);
+  }
 
   inline Value *get32BitLaneID(void) {
     Module *module = this->GetInsertBlock()->getModule();
@@ -204,6 +214,8 @@ public:
   Value *getRtMemBasePtr(Value *globalBufferPtr = nullptr);
   Value *getStackSizePerRay(void);
   Value *getNumDSSRTStacks(void);
+  Value *getBaseSurfaceStatePointer(void);
+  Value *getBaseSurfaceStatePointer(Value *RayDispatchGlobalData);
   Value *getMaxBVHLevels(void);
   Value *getStatelessScratchPtr(void);
   Value *getLeafType(StackPointerVal *StackPointer, Value *CommittedHit);
@@ -322,8 +334,10 @@ public:
   static bool checkAlign(Module &M, StructType *StructTy, uint32_t Align);
 
   Value *getGlobalSyncStackID();
+  Value *getGlobalSyncStackIDBase();
 
   Value *getGlobalDSSID();
+  uint32_t getRTStackSectorSize(uint32_t Align);
 
   Value *getSyncRTStackSize();
 
@@ -361,10 +375,6 @@ private:
   void setDereferenceable(CallInst *CI, uint32_t Size);
 
 
-private:
-#include "AutoGenRTStackReflectionPrivate.h"
-public:
-#include "AutoGenRTStackReflectionPublic.h"
 public:
   Type *getSMStack2Ty() const;
   Type *getRTStack2Ty() const;
