@@ -119,7 +119,17 @@ public:
   bool supportsBindlessSamplers() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN10_CORE; }
   bool supportsHDCLegacyDCROMessage() const { return m_platformInfo.eRenderCoreFamily <= IGFX_XE_HPC_CORE; }
 
+  bool hasEfficient64bEnabled() const {
+    return m_SkuTable.FtrEfficient64BitAddressing ||
+           (isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(EnableEfficient64b));
+  }
 
+  bool hasSWManagedStackCountEnabled() const {
+    return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_DISABLED(DisableSWManagedStack);
+  }
+
+
+  bool hasEngineIDEnabled() const { return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_DISABLED(DisableEngineID); }
   bool SupportSurfaceInfoMessage() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN9_CORE; }
   bool SupportHDCUnormFormats() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN10_CORE; }
   bool localMemFenceSupress() const {
@@ -187,7 +197,7 @@ public:
   bool isCoreXE2() const { return (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE); }
 
   bool isCoreXE3() const {
-    return (m_platformInfo.eRenderCoreFamily == IGFX_XE3_CORE);
+    return (m_platformInfo.eRenderCoreFamily == IGFX_XE3_CORE || m_platformInfo.eRenderCoreFamily >= IGFX_XE3P_CORE);
   }
 
   // This function checks if core is child of another core
@@ -263,6 +273,7 @@ public:
   unsigned int getMaxNumberThreadPerWorkgroupPooledMax() const {
     return m_caps.KernelHwCaps.EUCountPerPoolMax * m_caps.KernelHwCaps.EUThreadsPerEU;
   }
+  unsigned int getMaxSimdSize() const { return 32; }
   unsigned int getBarrierCountBits(unsigned int count) const {
     // Returns barrier count field + enable for barrier message
     if (m_platformInfo.eRenderCoreFamily <= IGFX_GEN10_CORE) {
@@ -314,8 +325,13 @@ public:
 
 
   unsigned int GetBindlessSamplerSize() const {
-    // Samplers are 16 bytes
-    return 16;
+    if (isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(Enable32bSampler)) {
+      // Samplers are 32 bytes
+      return 32;
+    } else {
+      // Samplers are 16 bytes
+      return 16;
+    }
   }
 
   unsigned int GetLogBindlessSamplerSize() const { return (unsigned int)iSTD::Log2(GetBindlessSamplerSize()); }
@@ -353,6 +369,8 @@ public:
   }
 
   bool supportsSourceModifierForMixedIntMad() const {
+    if (isCoreChildOf(IGFX_XE3P_CORE))
+      return true;
     return false;
   }
 
@@ -372,7 +390,7 @@ public:
            hasNoInt64Inst();
   }
 
-  bool HDCCoalesceSLMAtomicINCWithNoReturn() const { return m_platformInfo.eRenderCoreFamily >= IGFX_GEN12_CORE; }
+  bool HDCCoalesceSLMAtomicINCWithNoReturn() const { return m_platformInfo.eRenderCoreFamily == IGFX_GEN12_CORE; }
 
   int LSCCachelineSize() const { return 64; }
 
@@ -507,6 +525,12 @@ public:
 
   bool hasScratchSurface() const { return isProductChildOf(IGFX_XE_HP_SDV); }
 
+  bool has16MBPerThreadScratchSpace() const {
+    return isProductChildOf(IGFX_CRI);
+  }
+  bool canCoalesceAtomicWithNoReturn() const {
+    return isProductChildOf(IGFX_CRI);
+  }
 
   bool hasAtomicPreDec() const { return !isProductChildOf(IGFX_XE_HP_SDV); }
 
@@ -528,6 +552,10 @@ public:
 
   bool hasFP32GlobalAtomicAdd() const { return isProductChildOf(IGFX_XE_HP_SDV); }
 
+  bool hasFP32LocalAtomicAdd() const {
+    return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(EnableNativeFP32LocalAtomicAdd);
+  }
+
   bool has16OWSLMBlockRW() const {
     return IGC_IS_FLAG_ENABLED(Enable16OWSLMBlockRW) && isProductChildOf(IGFX_XE_HP_SDV);
   }
@@ -568,6 +596,7 @@ public:
     case IGFX_DG2:
     case IGFX_PVC:
     case IGFX_BMG:
+    case IGFX_CRI:
       return false;
     default:
       return true;
@@ -635,6 +664,10 @@ public:
            isCoreChildOf(IGFX_XE2_HPG_CORE);
   }
 
+  bool hasFP64DPAS() const { return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(EnableFP64Dpas); }
+
+  bool hasFP4DPAS() const { return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(EnableFP4Dpas); }
+
   bool hasExecSize16DPAS() const { return isCoreChildOf(IGFX_XE_HPC_CORE); }
 
   bool LSCSimd1NeedFullPayload() const {
@@ -650,6 +683,7 @@ public:
   bool hasBFTFDenormMode() const {
     return isCoreChildOf(IGFX_XE2_HPG_CORE);
   }
+  bool hasMullh() const { return isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(EnableMullh); }
 
   bool WaPredicatedStackIDRelease() const {
     return m_WaTable.Wa_22014559856 && IGC_IS_FLAG_DISABLED(DisablePredicatedStackIDRelease);
@@ -680,6 +714,7 @@ public:
   }
 
   bool supportRayTracing() const { return isProductChildOf(IGFX_DG2); }
+  bool supportRayTracingSIMD32() const { return isCoreChildOf(IGFX_XE3P_CORE); }
 
   bool isValidNumThreads(int32_t numThreadsPerEU) const {
     return numThreadsPerEU == 0 // "auto" mode - use compiler heuristic
@@ -691,7 +726,7 @@ public:
     return (m_platformInfo.eProductFamily != IGFX_XE_HP_SDV && m_platformInfo.eProductFamily != IGFX_DG2 &&
             m_platformInfo.eProductFamily != IGFX_METEORLAKE && m_platformInfo.eProductFamily != IGFX_ARROWLAKE &&
             m_platformInfo.eProductFamily != IGFX_BMG && m_platformInfo.eProductFamily != IGFX_LUNARLAKE &&
-            m_platformInfo.eRenderCoreFamily != IGFX_XE3_CORE
+            m_platformInfo.eRenderCoreFamily != IGFX_XE3_CORE && m_platformInfo.eRenderCoreFamily != IGFX_XE3P_CORE
     );
   }
 
@@ -851,9 +886,36 @@ public:
   // so it is not dependent on compiled SIMD size or "PreferredRayTracingSIMDSize".
   unsigned getRTStackDSSMultiplier() const {
     IGC_ASSERT(supportRayTracing());
+    if (hasEfficient64bEnabled() && IGC_IS_FLAG_DISABLED(DisableSWManagedStack)) {
+      unsigned int numSyncDssRTStacks = getsyncNumDSSRTStacks();
+
+      // 0 is special value used for backward compatibility.
+      // If number of SyncStacks, programmed in GlobalData is 0, HW assumes
+      // 2048 stacks are used.
+      if (numSyncDssRTStacks == 0) {
+        numSyncDssRTStacks = 2048;
+      }
+
+      IGC_ASSERT(numSyncDssRTStacks <= 4096);
+
+      return numSyncDssRTStacks;
+    } else
       return 2048;
   }
 
+  unsigned int getsyncNumDSSRTStacks() const {
+    if (hasEfficient64bEnabled() && IGC_IS_FLAG_DISABLED(DisableSWManagedStack)) {
+      if (IGC_IS_FLAG_SET(SWManagedStackNumStacks)) {
+        return IGC_GET_FLAG_VALUE(SWManagedStackNumStacks);
+      }
+
+      return GetGTSystemInfo().MaxEuPerSubSlice * (GetGTSystemInfo().ThreadCount / GetGTSystemInfo().EUCount) *
+             getMaxSimdSize();
+    }
+
+    return 0;
+  }
+
   // Max number of hw threads for each workgroup
   unsigned int getMaxNumberHWThreadForEachWG() const {
     if (m_platformInfo.eRenderCoreFamily < IGFX_GEN12_CORE) {
@@ -871,8 +933,10 @@ public:
       return getMaxNumberThreadPerSubslice();
     } else if (m_platformInfo.eRenderCoreFamily == IGFX_XE3_CORE) {
       return getMaxNumberThreadPerSubslice();
-    }
-    else {
+    } else if ((m_platformInfo.eRenderCoreFamily == IGFX_XE3P_CORE)
+    ) {
+      return getMaxNumberThreadPerSubslice();
+    } else {
       IGC_ASSERT_MESSAGE(0, "Unsupported platform!");
     }
     return 0;
@@ -881,16 +945,25 @@ public:
   uint32_t getGRFSize() const { return isCoreChildOf(IGFX_XE_HPC_CORE) ? 64 : 32; }
 
   uint32_t getMaxNumGRF(ShaderType type) const {
-      if (supportsVRT()) {
-        return 256;
-      } else {
-        return 128;
-      }
+    if (hasEfficient64bEnabled() &&
+        isProductChildOf(IGFX_CRI)) {
+      return (type == ShaderType::HULL_SHADER) ? 256 : 512;
+    } else if (supports512GRFPerThread() && type == ShaderType::OPENCL_SHADER) {
+      return 512;
+    } else if (supportsVRT()) {
+      return 256;
+    } else {
+      return 128;
+    }
   }
 
+  uint32_t getMinNumGRF() const { return supportsVRT() ? 32 : 128; }
+
   uint32_t getInlineDataSize() const {
     if (!supportInlineData())
       return 0;
+    if (hasEfficient64bEnabled())
+      return 64;
     return 32;
   }
 
@@ -914,6 +987,9 @@ public:
     auto immOffsetMode = (LscMatchImmMode)IGC_GET_FLAG_VALUE(LscImmOffsMatch);
     return hasLSC() && (immOffsetMode >= ON || (immOffsetMode == IF_HW_SUPPORTED && isCoreChildOf(IGFX_XE2_HPG_CORE)));
   }
+  bool supportStatefulScaleFolding() const {
+    return hasEfficient64bEnabled() && IGC_IS_FLAG_ENABLED(EnableStatefulScaleFolding);
+  }
 
   bool WaCubeHFPrecisionBug() const { return m_WaTable.Wa_18012201914 != 0; }
 
@@ -921,10 +997,15 @@ public:
   //   XeHP_SDV and above are for each physical thread: 256k.
   //   TGLLP and below are for each FFTID: 2M.
   uint32_t maxPerThreadScratchSpace(
-  ) const {
-    if(IGC_IS_FLAG_ENABLED(MaxPerThreadScratchSpaceOverride)){
+      // This is a temporary solution for the enabling.
+      // It should be removed when all UMDs implement
+      // support for 16MB scratch.
+      bool sixteenMBPerThreadScratchSpace) const {
+    if (IGC_IS_FLAG_ENABLED(MaxPerThreadScratchSpaceOverride)) {
       return IGC_GET_FLAG_VALUE(MaxPerThreadScratchSpaceOverride);
     }
+    if (sixteenMBPerThreadScratchSpace && has16MBPerThreadScratchSpace())
+      return 0x1000000;
 
     return hasScratchSurface() ? 0x40000 : 0x200000;
   }
@@ -977,6 +1058,7 @@ public:
             m_platformInfo.eProductFamily != IGFX_DG1 && m_platformInfo.eProductFamily != IGFX_ALDERLAKE_S &&
             m_platformInfo.eProductFamily != IGFX_ALDERLAKE_P && m_platformInfo.eProductFamily != IGFX_ALDERLAKE_N &&
             m_platformInfo.eProductFamily != IGFX_DG2 && m_platformInfo.eRenderCoreFamily != IGFX_XE3_CORE &&
+            m_platformInfo.eRenderCoreFamily != IGFX_XE3P_CORE &&
             m_platformInfo.eProductFamily != IGFX_METEORLAKE) &&
            m_platformInfo.eProductFamily != IGFX_ARROWLAKE && m_platformInfo.eProductFamily != IGFX_BMG &&
            m_platformInfo.eProductFamily != IGFX_LUNARLAKE;
@@ -1086,13 +1168,16 @@ public:
     return enabled;
   }
 
-  bool needsLocalScopeEvictTGM() const { return IGC_IS_FLAG_ENABLED(ForceLocalScopeEvictTGM); }
+  bool needsLocalScopeEvictTGM() const {
+    return
+            true;
+  }
 
   bool needWaSamplerNoMask() const { return m_WaTable.Wa_22011157800 && !IGC_IS_FLAG_DISABLED(DiableWaSamplerNoMask); }
 
   bool hasSlowSameSBIDLoad() const {
     bool bYes = false;
-    bYes = isCoreChildOf(IGFX_XE_HPG_CORE);
+    bYes = isCoreChildOf(IGFX_XE_HPG_CORE) && !isCoreChildOf(IGFX_XE3P_CORE);
 
     return bYes && !needWaSamplerNoMask();
   }
@@ -1104,6 +1189,7 @@ public:
   }
 
   bool supportsProgrammableOffsets() const { return isCoreChildOf(IGFX_XE2_HPG_CORE); }
+  bool supportsProgrammableOffsetsSampleBCAndSampleDC() const { return isCoreChildOf(IGFX_XE3P_CORE); }
 
   bool supportsRayQueryThrottling() const { return isCoreChildOf(IGFX_XE2_HPG_CORE); }
 
@@ -1114,11 +1200,16 @@ public:
     if (IGC_IS_FLAG_SET(OverrideRayQueryThrottling))
       return IGC_GET_FLAG_VALUE(OverrideRayQueryThrottling);
 
+    if (isCoreChildOf(IGFX_XE3P_CORE))
+      return true;
+
     return enableByDefault;
   }
 
   bool isSWSubTriangleOpacityCullingEmulationEnabled() const {
-    return (isCoreChildOf(IGFX_XE3_CORE) && IGC_IS_FLAG_DISABLED(DisableSWSubTriangleOpacityCullingEmulation));
+    return (isCoreChildOf(IGFX_XE3_CORE) &&
+            IGC_IS_FLAG_DISABLED(DisableSWSubTriangleOpacityCullingEmulation)
+    );
   }
 
   bool isRayQueryReturnOptimizationEnabled() const {
@@ -1263,8 +1354,13 @@ public:
 
   bool supportCheckCSThreadsLimit() const { return (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE); }
   bool supportTriggerLargeGRFRetry() const { return (m_platformInfo.eRenderCoreFamily == IGFX_XE2_HPG_CORE); }
+  bool supports512GRFPerThread() const { return isCoreChildOf(IGFX_XE3P_CORE); }
+  bool supportsQuantumDispatch() const { return isCoreChildOf(IGFX_XE3P_CORE); }
 
-
+  bool supportsPureBF() const {
+    // Maps to vISA's "supportPureBF"
+    return isCoreChildOf(IGFX_XE3P_CORE);
+  }
 
   bool EnableCSWalkerPass() const { return isCoreChildOf(IGFX_XE2_HPG_CORE); }
 
@@ -1283,19 +1379,23 @@ public:
   bool supportsNumberOfBariers() const { return isProductChildOf(IGFX_DG2); }
 
   uint32_t getVISAABIVersion() const {
+    if (isCoreChildOf(IGFX_XE3P_CORE))
+      return 3;
     return 2;
   }
 
-bool supportsLoadStatusMessages() const {
-    return
-        isCoreChildOf(IGFX_XE2_HPG_CORE);
-  }
+  bool supportsLoadStatusMessages() const { return isCoreChildOf(IGFX_XE2_HPG_CORE); }
 
   bool supportsNonDefaultLSCCacheSetting() const {
     return isCoreChildOf(IGFX_XE2_HPG_CORE);
   }
 
+  bool hasNewLSCCacheEncoding() const {
+    return isCoreChildOf(IGFX_XE3P_CORE);
+  }
   bool isSupportedLSCCacheControlsEnum(LSC_L1_L3_CC l1l3cc, bool isLoad) const {
+    if (hasNewLSCCacheEncoding())
+      return true;
     if (isLoad) {
       switch (l1l3cc) {
       default:
@@ -1313,6 +1413,10 @@ bool supportsLoadStatusMessages() const {
     const DWORD blockSize = getSharedLocalMemoryBlockSize() ? getSharedLocalMemoryBlockSize() : sizeof(KILOBYTE);
     size = iSTD::Round(size, blockSize) / blockSize;
     if (size > 16) {
+      if (isCoreChildOf(IGFX_XE3P_CORE) && size > 256) {
+        // 320K does not follow Xe2 pattern
+        return (size > 320) ? 384 * blockSize : 320 * blockSize;
+      }
       if (isCoreChildOf(IGFX_XE2_HPG_CORE)) {
         const DWORD roundSize = iSTD::RoundPower2(size) / 4;
         return iSTD::Round(size, roundSize) * blockSize;
@@ -1321,6 +1425,41 @@ bool supportsLoadStatusMessages() const {
     return iSTD::RoundPower2(size) * blockSize;
   }
 
+  bool supports2dBlockTranspose64ByteWidth() const { return isCoreChildOf(IGFX_XE3P_CORE); }
+
+  bool supportsReadStateInfo() const {
+    return hasEfficient64bEnabled() && (IGC_IS_FLAG_ENABLED(EnableReadStateToA64Read) ||
+        m_WaTable.Wa_14025275057);
+  }
+
+  bool supportsFp4Int4Upsampling() const { return isCoreChildOf(IGFX_XE3P_CORE); }
+
+
+  bool supportsRayTracingExtendedCacheControl() const {
+    return isProductChildOf(IGFX_CRI) && hasEfficient64bEnabled() &&
+           IGC_IS_FLAG_DISABLED(DisableRayTracingExtendedCacheControl);
+  }
+
+
+  bool supportsOverfetch() const {
+    return isProductChildOf(IGFX_CRI);
+  }
+
+  bool supportsNativeHyperbolicTan() const { return isCoreChildOf(IGFX_XE3P_CORE); }
+
+  bool supportsNativeTanh() const { return supportsNativeHyperbolicTan() && IGC_IS_FLAG_ENABLED(EnableNativeTanh); }
+
+  bool supportsNativeSigmoid() const { return isCoreChildOf(IGFX_XE3P_CORE); }
+
+  bool supportsNativeSinCos() const {
+    return isProductChildOf(IGFX_CRI) && IGC_IS_FLAG_ENABLED(EnableNativeSinCos);
+  }
+
+  bool hasAccurateLog2() const {
+    return isProductChildOf(IGFX_CRI);
+  }
+
+
   unsigned getRayTracingTileXDim1D() const {
     if (isCoreChildOf(IGFX_XE2_HPG_CORE)) {
       return 512;
@@ -1360,11 +1499,11 @@ bool supportsLoadStatusMessages() const {
   bool allowsMoviForType(VISA_Type type) const { return (type == ISA_TYPE_UD || type == ISA_TYPE_D); }
 
   bool enableLscSamplerRouting() const {
-      return isCoreChildOf(IGFX_XE3_CORE);
+    return isCoreChildOf(IGFX_XE3_CORE);
   }
 
   bool enableReplaceAtomicFenceWithSourceValue() const {
-    return IGC_IS_FLAG_ENABLED(ReplaceAtomicFenceWithSourceValue);
+    return !isCoreChildOf(IGFX_XE3P_CORE) && IGC_IS_FLAG_ENABLED(ReplaceAtomicFenceWithSourceValue);
   }
 };
 } // namespace IGC
