@@ -23,6 +23,7 @@ SPDX-License-Identifier: MIT
 #include <string.h>
 #include <sstream>
 #include <unordered_map>
+#include <unordered_set>
 #include "Compiler/CISACodeGen/ShaderUnits.hpp"
 #include "Compiler/CISACodeGen/Platform.hpp"
 #include "Compiler/CISACodeGen/DriverInfo.hpp"
@@ -504,6 +505,9 @@ struct SBindlessProgram : SKernelProgram {
   // This is just diagnostic information to track whether we picked
   // the retry shader
   bool BetterThanPrev = false;
+
+  // Informs if the kernel requires extra allocation of the sync HW stack
+  bool UsesSyncHWStack = false;
 };
 
 struct SRayTracingShadersGroup {
@@ -520,6 +524,8 @@ struct SRayTracingShadersGroup {
   BindlessShaderVec m_CallableShaders;
   // Continuation shaders
   BindlessShaderVec m_Continuations;
+
+  std::optional<SBindlessProgram> prologueKernel;
 };
 
 struct SRayTracingPipelineConfig {
@@ -934,8 +940,12 @@ public:
   std::vector<std::string> entry_names;
   uint m_spillAllowed = 0;
   uint m_spillAllowedFor256GRF = 0;
+  uint m_ForceSIMDRPELimit = 0;
 
 private:
+  // For storing emitted warning messages
+  std::unordered_set<std::string_view> m_emittedWarnings;
+
   // For storing error message
   std::stringstream oclErrorMessage;
   // For storing warning message
@@ -982,6 +992,7 @@ public:
 
     // Set retry behavor for Disable()
     m_retryManager.perKernel = (type == ShaderType::OPENCL_SHADER);
+    m_ForceSIMDRPELimit = IGC_GET_FLAG_VALUE(ForceSIMDRPELimit);
   }
 
   CodeGenContext(CodeGenContext &) = delete;
@@ -1148,8 +1159,10 @@ public:
   void initializeRemarkEmitter(const ShaderHash &hash);
 
   bool syncRTCallsNeedSplitting() {
-       // In general, we don't want to compile SIMD32 for rayquery.
-       // Determine if we are forced to do so.
+    if (platform.supportRayTracingSIMD32())
+      return false;
+    // In general, we don't want to compile SIMD32 for rayquery.
+    // Determine if we are forced to do so.
 
     if (type != ShaderType::COMPUTE_SHADER)
       return false;
@@ -1179,13 +1192,13 @@ public:
 
   bool useStatelessToStateful() {
     return (m_instrTypes.hasLoadStore && m_DriverInfo.SupportsStatelessToStatefulBufferTransformation() &&
-            !getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired &&
+            !getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired && !platform.hasEfficient64bEnabled() &&
             IGC_IS_FLAG_ENABLED(EnableStatelessToStateful) && !m_instrTypes.hasInlineAsmPointerAccess);
   }
 
   bool supportsVRT() const {
     return platform.supportsVRT() && m_DriverInfo.supportsVRT() &&
-           (getModuleMetaData()->compOpt.EnableVRT && IGC_IS_FLAG_ENABLED(EnableVRT));
+           (getModuleMetaData()->compOpt.EnableVRT || IGC_IS_FLAG_ENABLED(EnableVRT));
   }
 };
 
